/*
 * cap: trivial cap tool
 *
 * GPL v2
 * Copyright (C) 2004 Andy Lutomirski
 */

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/capability.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/prctl.h>
#include <pwd.h>
#include <grp.h>

void barf(const char *err)
{
	if(err) printf("%s\n", err);
	printf("Usage: cap options program args\n");
	printf(" -u user\n -g group\n -e effective only\n"
	       " -a dump AUXV table\n -c caps\n");
	exit(1);
}

int main(int argc, char **argv)
{
	int douser = 0, dogroup = 0, doeffective = 0, doauxv = 0;
	char *caps = NULL;
	uid_t uid;
	gid_t gid;
	while(1) {
		int c = getopt(argc, argv, "+c:u:g:ae");
		if(c==-1) break;
		switch(c)
		{
		case 'c':
			caps = optarg;
			break;

		case 'u':
			douser = dogroup = 1;
			gid = uid = atoi(optarg);
			break;
		case 'g':
			dogroup = 1;
			gid = atoi(optarg);
			break;
		case 'e':
			doeffective = 1;
			break;
		case 'a':
			doauxv = 1;
			break;
		default:
			barf(NULL);
			break;
		}
	}

	if(optind >= argc)
		barf("Not enough arguments");

	if(0 != prctl(PR_SET_KEEPCAPS, 1, 0, 0, 0))
		barf("Couldn't set keepcaps");

	if(dogroup) {
		if (doeffective) {
			if(0 != setegid(gid))
				barf("Couldn't set egid");
		} else {
			if(0 != setresgid(gid, gid, gid))
				barf("Couldn't set gid");
		}
	}
	if(douser) {
		if (doeffective) {
			if(0 != seteuid(uid))
				barf("Couldn't set euid");
		} else {
			if(0 != setresuid(uid, uid, uid))
				barf("Couldn't set uid");
		}
	}

	if(caps) {
		cap_t newcaps;
		cap_t mycaps = cap_get_proc();
		char *mycaps_str = cap_to_text(mycaps, NULL);
	
		char capbuf[1024];
		ssize_t capsize;
		strcpy(capbuf, mycaps_str);
		cap_free(mycaps);
		cap_free(mycaps_str);
		strcat(capbuf, " ");
		strcat(capbuf, caps);
		newcaps = cap_from_text(capbuf);
		if(!newcaps) {
			perror("cap_from_text");
			barf(0);
		}

		if(0 != cap_set_proc(newcaps))
			barf("Couldn't set caps");

		cap_free(newcaps);
	}

	if (doauxv) {
		setenv("LD_SHOW_AUXV", "1", 1);
	}
#if 0
	if (dofork) {
		int pid = fork();
		if (pid == 0)
			goto exec;
		else if (pid > 0) {
			waitpid(pid, NULL, 0);
			exit(0);
		}
		else
			barf("fork");
	}
#endif
exec:
	execvp(argv[optind], argv + optind);
	perror("execvp");
	return errno;
}
