#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/prctl.h>
#include <sys/capability.h>


void usage(void)
{
	printf("Usage: dropuid options\n");
	printf(" -u uid\n -e drop effective first\n -k keepcaps\n"
	       " -x command to exec\n");
}

void dumpcred(void)
{
	uid_t ruid, euid, suid;
	gid_t rgid, egid, sgid;
	getresuid(&ruid, &euid, &suid);
	getresgid(&rgid, &egid, &sgid);
	printf("        Real        Eff        Saved\n");
	printf("User    %-12d%-12d%-12d\n", ruid, euid, suid);
	printf("Group   %-12d%-12d%-12d\n\n", rgid, egid, sgid);

	cap_t caps = cap_get_proc();
	if(caps)
	{
		char *ct = cap_to_text(caps, NULL);
		if(ct) {
			printf("Caps: %s\n", ct);
			cap_free(ct);
		}
		cap_free(caps);
	}
}

main(int argc, char *argv[])
{
	uid_t new_uid = 500;
	int do_effective = 0, keepcaps=0, do_exec=0;

	while(1) {
		int c = getopt(argc, argv, "+u:ekx");
		if(c==-1) break;
		switch(c)
		{
		case 'u':
			new_uid = atoi(optarg);
			break;
		case 'e':
			do_effective = 1;
			break;
		case 'k':
			keepcaps = 1;
			break;
		case 'x':
			do_exec = 1;
			break;
		default:
			usage();
			exit(1);
			break;
		}
	}
	dumpcred();
	prctl(PR_SET_KEEPCAPS, keepcaps);
	if (do_effective) {
		if (seteuid(new_uid) == -1)
			printf("seteuid: %s\n", strerror(errno));
	} else {
		if (setuid(new_uid) == -1)
			printf("setuid: %s\n", strerror(errno));
	}
	if (do_exec) {
		execvp(argv[optind], argv+optind);
		perror("execvp");
		exit(1);
	}
	dumpcred();
}
