/*
 * dumpcap: trivial cap tool
 * GPL v2
 * Copyright (c) 2004 Andy Lutomirski
 */
#include <sys/types.h>
#include <sys/capability.h>
#include <unistd.h>
#include <stdio.h>

void usage(void)
{
	printf("Usage: dumpcap options\n");
	printf(" -x command to exec\n");
}

void dumpcred(void)
{
	uid_t ruid, euid, suid;
	gid_t rgid, egid, sgid;
	getresuid(&ruid, &euid, &suid);
	getresgid(&rgid, &egid, &sgid);
	fprintf(stdout, "        Real        Eff        Saved\n");
	fprintf(stdout, "User    %-12d%-12d%-12d\n", ruid, euid, suid);
	fprintf(stdout, "Group   %-12d%-12d%-12d\n\n", rgid, egid, sgid);

	cap_t caps = cap_get_proc();
	if(caps)
	{
		char *ct = cap_to_text(caps, NULL);
		if(ct) {
			fprintf(stdout, "Caps: %s\n", ct);
			cap_free(ct);
		}
		cap_free(caps);
	}
	fflush(stdout);
}

int main(int argc, char *argv[])
{
	int do_exec = 0;
	while(1) {
		int c = getopt(argc, argv, "x");
		if(c==-1) break;
		switch(c)
		{
		case 'x':
			do_exec = 1;
			break;
		default:
			usage();
			exit(1);
			break;
		}
	}
	dumpcred();
	if (do_exec) {
		execvp(argv[optind], argv+optind);
		perror("exec");
		exit(1);
	}
}
