# Tests for dropping capabilities
#
do_test()
{
  _num=`printf "%02d" $1`
  shift
  echo "BEGIN test ${_num}" > testcap_${_num}.out
  "$@" >> testcap_${_num}.out 2>&1
  echo -e "END test ${_num}\n" >> testcap_${_num}.out
}

echo -e "Capabilities test run for uid $UID\n"
#
# Test the following without changing inheritable:
# 1 Root execs root process, no caps change (caps are preserved)
# 2 Non-root execs non root process, no caps change (caps aren't gained/lost)
# 3 Root execs setuid-root process, no cap change (retains full caps)
# 4 Root execs setuid-nonroot process, effective caps are dropped
# 5 Non-root execs setuid-root process, full caps are gained
# 6 Non-root execs setuid-nonroot process, no caps change
# 7 Non-root execs setuid-root which execs setuid-back, no caps are gained
#
if [ $UID -eq 0 ]; then
  do_test 1 ./dumpcap
else
  do_test 2 ./dumpcap
fi

if [ $UID -eq 0 ]; then
  do_test 3 ./sdumpcap
else
  do_test 5 ./sdumpcap
fi

if [ $UID -eq 0 ]; then
  do_test 4 ./sbindumpcap
else
  do_test 6 ./sbindumpcap
  do_test 7 ./sdumpcap -x -- ./s500dumpcap
fi
#
# Still w/out changing inheritable and w/out KEEPCAPS set
# 8 Root process does setuid(!0), all caps are dropped
# 9 Root process does seteuid(!0), effective caps are dropped
# 10 Nonroot process restores uid 0, effective restored to permitted
#
if [ $UID -eq 0 ]; then 
  do_test 8 ./dropuid -u 50
fi

if [ $UID -eq 0 ]; then 
  do_test 9 ./dropuid -u 50 -e
fi

if [ $UID -ne 0 ]; then
  do_test 10 sudo ./cap -u 50 -e ./dropuid -u 0
fi
#
# Still w/out changing inheritable and with KEEPCAPS set
# 11 Root process does setuid(!0), effective caps are dropped
# 12 Root process does seteuid(!0), effective caps are dropped
# 13 Nonroot process restores uid 0, effective restored to permitted
#
if [ $UID -eq 0 ]; then
  do_test 11 ./dropuid -u 50 -k
fi

if [ $UID -eq 0 ]; then
  do_test 12 ./dropuid -u 50 -e -k
fi

if [ $UID -ne 0 ]; then
  do_test 13 sudo ./cap -u 50 -e ./dropuid -u 0 -k
fi
#
# Test the following with a limited inheritable set:
# 14 Root execs root process, new caps bound by inheritable
# 15 Non-root execs non root process, new caps bound by inheritable
# 16 Root execs setuid-root process, full caps are gained
# 17 Root execs setuid-nonroot process, new caps bound by inh, drop eff
# 18 Non-root execs setuid-root process, full caps are gained
# 19 Non-root execs setuid-nonroot process, new caps bound by inheritable
# 20 Non-root execs setuid-root which execs setuid-back, no caps are gained
#
if [ $UID -eq 0 ]; then
  do_test 14 ./cap -c "= cap_ipc_lock+eip" ./dumpcap
else
  do_test 15 sudo ./cap -c "= cap_ipc_lock+eip" -u 500 ./dumpcap
fi

if [ $UID -eq 0 ]; then
  do_test 16 ./cap -c "= cap_ipc_lock+eip" ./sdumpcap
else
  do_test 18 sudo ./cap -c "= cap_ipc_lock+eip" -u 500 ./sdumpcap
fi

if [ $UID -eq 0 ]; then
  do_test 17 ./cap -c "= cap_ipc_lock+eip" ./sbindumpcap
else
  do_test 19 sudo ./cap -c "= cap_ipc_lock+eip" -u 500 ./sbindumpcap
  do_test 20 sudo ./cap -c "= cap_ipc_lock+eip" -u 500 ./sdumpcap -x -- ./s500dumpcap
fi
#
# With limited inheritable and w/out KEEPCAPS set
# 21 Root process (w/ CAP_SETUID) does setuid(!0), all eff/perm caps are dropped
# 22 Root process (w/ CAP_SETUID) does seteuid(!0), effective caps are dropped
# 23 Nonroot process restores uid 0, effective restored to permitted
#
if [ $UID -eq 0 ]; then
  do_test 21 ./cap -c "= cap_ipc_lock+eip cap_setuid+eip" ./dropuid -u 50
fi

if [ $UID -eq 0 ]; then
  do_test 22 ./cap -c "= cap_ipc_lock+eip cap_setuid+eip" ./dropuid -u 50 -e
fi

if [ $UID -ne 0 ]; then
  do_test 23 sudo ./cap -c "= cap_ipc_lock+ip" -u 50 -e ./dropuid -u 0 
fi
#
# With limited inheritable and with KEEPCAPS set
# 24 Root process (w/ CAP_SETUID) does setuid(!0), effective caps are dropped
# 25 Root process (w/ CAP_SETUID) does seteuid(!0), effective caps are dropped
# 26 Nonroot process restores uid 0, effective restored to permitted
#
if [ $UID -eq 0 ]; then
  do_test 24 ./cap -c "= cap_ipc_lock+eip cap_setuid+eip" ./dropuid -u 50 -k
fi

if [ $UID -eq 0 ]; then
  do_test 25 ./cap -c "= cap_ipc_lock+eip cap_setuid+eip" ./dropuid -u 50 -e -k
fi

if [ $UID -ne 0 ]; then
  do_test 26 sudo ./cap -c "= cap_ipc_lock+ip" -u 50 -e ./dropuid -u 0 -k
fi
#
# Capset inheritable
# 27 Root Can't set inheritable to -1 (reserved for legacy mode)
# 28 Can't set inheritable to -1 (reserved for legacy mode)
#
if [ $UID -eq 0 ]; then
  do_test 27 ./cap -c "=eip" ./dumpcap
else
  do_test 28 ./cap -c "=eip" ./dumpcap
fi
#
# Ptrace interaction
# 29 Can't elevate privs while ptracing a setuid-root app (w/out change to inh)
# 30 Can't elevate privs while ptracing a setuid-root app (w/ change to inh)
# 31 Can't elevate privs while ptracing a setuid-root app (w/ change to inh)
#
if [ $UID -ne 0 ]; then
  do_test 29 strace -o /dev/null ./sdumpcap
  do_test 30 sudo ./cap -c "= cap_ipc_lock+eip" -u 500 strace -o /dev/null ./sdumpcap
else
  do_test 31 ./cap -c "= cap_ipc_lock+eip" strace -o /dev/null ./sdumpcap
fi
#
# make sure to honor MNT_NOSUID
# 32 Non-root execs setuid-root under nosuid mntpoint, no caps gained
# 33 Non-root w/ some caps execs setuid-root under nosuid mntpoint, no caps
#    gained
# 34 Root w/ limited caps execs setuid-root under nosuid mntpoint, no caps
#    gained
# 35 Root w/ limited caps execs setuid-non-root under nosuid mntpoint, no caps 
#    gained
#
if [ $UID -ne 0 ]; then
  sudo mount --bind ./sdumpcap ./sdumpcap
  sudo mount -oremount,nosuid ./sdumpcap
  do_test 32 ./sdumpcap
  do_test 33 sudo ./cap -c"= cap_ipc_lock+eip" -u 500 ./sdumpcap
  sudo umount ./sdumpcap
else
  mount --bind ./sdumpcap ./sdumpcap
  mount -oremount,nosuid ./sdumpcap
  do_test 34 ./cap -c"= cap_ipc_lock+eip" ./sdumpcap
  umount ./sdumpcap
  mount --bind ./s500dumpcap ./s500dumpcap
  mount -oremount,nosuid ./s500dumpcap
  do_test 35 ./cap -c"= cap_ipc_lock+eip" ./s500dumpcap
  umount ./s500dumpcap
fi
#
# set AT_SECURE properly
# 36 Non-root user execs setuid-root, AT_SECURE is set
# 37 Root user w/ limited caps execs setuid-root, AT_SECURE is set
# 
if [ $UID -ne 0 ]; then
  do_test 36 sudo ./cap -c "= cap_ipc_lock+eip" -u 500 -a ./sdumpcap
else
  do_test 37 ./cap -c "= cap_ipc_lock+eip" -a ./sdumpcap
fi
